/**
 * Copyright (c) 2009, Opera Software ASA
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Opera Software ASA nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY OPERA SOFTWARE ASA AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL OPERA SOFTWARE ASA AND CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
 
/**
 * @fileoverview
 * Yusef Activity Feed plugin
 *
 * @author David Håsäther (davidh@opera.com)
 * @version 0.1
 */
Yusef.addPlugin
(
    "activityFeed",
    /**
     */
    function Yusef_plugins_activityFeed()
    {
        var self = this;
        var SERVICES_CHECK_REFRESH = 3600; // in seconds
        /**
         * @constant URL path where this plugin specific files are exposed
         */
        this.PLUGIN_PATH = 'yusef_plugins_activityFeed';
        /**
         * @constant Id used to publish content on the {@link PSO} object.
         */
        var ERR = 'Yusef.plugins.activityFeed';
        
        this.feeds = getSavedFeeds();
        
        this.getData = function( connection )
        {
            var feeds = getSavedFeeds();
            var entries = [];

            // flatten
            for (var feedId in feeds)
            {
                var feedEntries = feeds[feedId].entries;
                for (var i = 0, entry; entry = feedEntries[i]; i++)
                {
                    if( !isActivityFeedEntry(entry) ) { continue; }
                    
                    var text = "";
                    var variables = {
                        "link": entry.links[0].href,
                        "relativeTime": entry.published.parseRfcDate().happens(),
                        "appLink": feedId.replace("http://", "http://admin."),
                        "appName": feeds[feedId].title
                    };
                    switch (entry.object["object-type"])
                    {
                    case "file":
                        variables.filename = entry.object.title;
                        text = _("<A href=\"{link}\">{filename}</A> was viewed in <A href=\"{appLink}\">{appName}</A> {relativeTime}.", variables);
                        break;
                    case "photo":
                        variables.filename = entry.object.title;
                        text = _("<A href=\"{link}\">{filename}</A> was viewed in <A href=\"{appLink}\">{appName}</A> {relativeTime}.", variables);
                        break;
                    case "audio":
                        variables.filename = entry.object.title;
                        text = _("<A href=\"{link}\">{filename}</A> was played by <A href=\"{appLink}\">{appName}</A> {relativeTime}.", variables);
                        break;
                    case "note":
                        variables.userName = entry.object.title;
                        text = _("<A href=\"{link}\">{userName}</A> posted a note on <A href=\"{appLink}\">{appName}</A> {relativeTime}.", variables);
                        break;
                    case "person":
                        variables.userName = entry.object.title;
                        text = _("<A href=\"{link}\">{userName}</A> joined <A href=\"{appLink}\">{appName}</A> {relativeTime}.", variables);
                        break;
                    default:
                        prefix = "";
                    }

                    entries.push({
                        appName: feeds[feedId].title,
                        text: text,
                        published: entry.published.parseRfcDate().getTime()
                    });
                }
            }

            // sort
            entries.sort(function(a, b) {
                return b.published - a.published;
            });
            
            return {
                entries: entries.slice(0, 100),
                recentEntries: entries.slice(0, 5),
                hasEntries: (entries.length>0),
                activityPage: connection.request.uri.indexOf("activities")>-1
            };
        }

        function isActivityFeedEntry(entry)
        {
            return typeof(entry) == 'object'
                && ('published' in entry)
                && typeof(entry.published.parseRfcDate) == 'function'
                && ('object' in entry)
                && ('links' in entry);
        }

        function getSavedFeeds()
        {
            var feeds;
            try {
                feeds = JSON.parse(widget.preferenceForKey("yusef.plugins.activityFeed.feeds"));
            }
            catch (ex)
            {
                feeds = {};
            }
            return feeds;
        }

        // FIXME: needs base tag awareness.
        function findPageActivityFeed( url, callback )
        {
            var xhr = new XMLHttpRequest();
            var baseUrl = url+'/';
            
            xhr.onreadystatechange = function()
            {
                if( xhr.readyState == 4 && xhr.status == 200 )
                {
                    var html = xhr.responseText;
                    var regexp = /<link.+?type=("|')application\/atom\+xml(\1).*?>/ig;
                    var match;
                    var feedUrl;
                    
                    while( match = regexp.exec(html) )
                    {
                        /href=("|')(.*?)\1/i.exec(match[0]);
                        feedUrl = RegExp.$2;
                        if( feedUrl[0] == '/' )
                        {
                            baseUrl = url.split('/').slice(0, -1).join('/');
                        }
                        //PSO.pub( 'afonso', baseUrl+feedUrl );
                        if( callback && feedUrl ) { callback(baseUrl+feedUrl); }
                    }
                }
            };
            xhr.open("GET", url);
            try {
                xhr.send(null);
            } catch( ex ) {
                opera.postError( "Couldn't connect to url: " + url + "\nException: " + ex );
            }
        }
        
        function fetchActivityStreams()
        {
            var fetcher = new ActivityStreamFetcher();
            fetcher.addCallback(onResourceUpdated);
            var findServicesFeeds = function()
            {
                var apps = opera.io.webserver.services;
                for (var i = 0, app; app = apps[i]; i++)
                {
                    findPageActivityFeed( app.uri, function(feedUrl)
                    {
                        // only add new url's
                        if( fetcher.getStatus(feedUrl).length == 0 )
                        {
                            fetcher.addUrl(feedUrl);
                        }
                    });
                }
            };
            
            setTimeout( findServicesFeeds, 0 );
            setInterval
            (
                findServicesFeeds,
                SERVICES_CHECK_REFRESH * 1000
            );

            function onResourceUpdated(url, data)
            {
                if (!data) { return; }
                self.feeds[data.id] = data;
                widget.setPreferenceForKey(JSON.stringify(self.feeds), "yusef.plugins.activityFeed.feeds");
            }
        };

        fetchActivityStreams();

        Yusef.addSectionListener
        (
            'activities',
            (function(connection)
            {
                if (!connection.isOwner)
                {
                    Yusef.emitErrorPage( connection );
                    return;
                }

                var data = {};
                data.entries = self.getData(connection).entries;

                var template = new Markuper("libraries/yusef/plugins/activityFeed/activities.html", data);

                // FIXME: this should not be duplicated here
                var tabs = [
                    {title: _("Applications"), isRoot: true}/*,
                    {title: "Account", hideFromVisitors: true}*/
                ];

                return {
                     content: template.parse(data).select(),
                     styles: [ { href: "style.css" } ],
                     tabs: Yusef.plugins.ui.generateTabs(tabs, connection)
                };
            }).bind(this),
            {"ui": {"activities": true}}
        );
    }
);
