/**
 * Copyright (c) 2009, Opera Software ASA
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Opera Software ASA nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY OPERA SOFTWARE ASA AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL OPERA SOFTWARE ASA AND CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
 
/**
 * @fileoverview
 * Yusef Profile Plugin.
 *
 * @author David Håsäther (davidh@opera.com)
 * @version 0.1
 */
Yusef.addPlugin
(
    "profile",
    /**
     */
    function Yusef_plugins_profile()
    {
        var self = this;
        /**
         * @constant URL path where this plugin specific files are exposed
         */
        this.PLUGIN_PATH = 'yusef_plugins_profile';
        /**
         * @constant Id used to publish content on the {@link PSO} object.
         */
        var ERR = 'Yusef.plugins.profile';

        var DEFAULT_STATUS = "This page lists applications running on {username}'s computer. Click for direct access to the content.";

        function redirectAndGet(connection, url)
        {
            connection.response.setStatusCode("303", "See Other");
            connection.response.setResponseHeader("Location", url);
            connection.response.close();
            return;
        }

        this.getData = function()
        {
            return {
                name: widget.preferenceForKey("yusef.plugins.profile.account.name"),
                email: widget.preferenceForKey("yusef.plugins.profile.account.email"),
                country: widget.preferenceForKey("yusef.plugins.profile.account.country"),
                status: widget.preferenceForKey("yusef.plugins.profile.status") || _(DEFAULT_STATUS, {username: opera.io.webserver.userName}),
                hasStatus: (widget.preferenceForKey("yusef.plugins.profile.status") != "")
            }
        }

        this.formatData = function(data)
        {
            try
            {
                var tmpl = new Markuper("libraries/yusef/plugins/profile/profile.xml", data);
                return tmpl.parse(data).xml();
            }
            catch(ex)
            {
                return ex.message;
            }
        }

        Yusef.addSectionListener
        (
            "profile",
            function(connection)
            {
                connection.response.setResponseHeader("Content-Type", "application/xml")
                return self.formatData(self.getData());
            }
        );

        /**
         *
         */
        Yusef.registerUniteActionListener
        (
            "yusef.plugins.profile.status.update",
            function(connection)
            {
                var status = (connection.request.bodyItems["status"]||[])[0] || "";

                widget.setPreferenceForKey(status.slice(0, 254), "yusef.plugins.profile.status");

                redirectAndGet(connection, "http://" + connection.request.headers.Host[0] + connection.request.uri);
                return;
            }
        );

        /**
         *
         */
        Yusef.registerUniteActionListener
        (
            "yusef.plugins.profile.account.update",
            function(connection)
            {
                // check password

                var req = connection.request;
                var name = (req.bodyItems["name"]||[])[0] || "";
                var email = (req.bodyItems["email"]||[])[0] || "";
                var country = (req.bodyItems["country"]||[])[0] || "";
                var password = (req.bodyItems["password"]||[])[0] || "";

                widget.setPreferenceForKey(name, "yusef.plugins.profile.account.name");
                widget.setPreferenceForKey(email, "yusef.plugins.profile.account.email");
                widget.setPreferenceForKey(country, "yusef.plugins.profile.account.country");

                redirectAndGet(connection, "http://" + connection.request.headers.Host[0] + connection.request.uri);
                return;
            }
        );
    }
);
